/**************************************************************************
Security.c
A7 Engineering, 2004

This program is used with RCM3000, RCM3100, RCM3200, RCM3300, and
RCM3400 series controllers with an EmbeddedBlue eb506 radio module.

Description
===========
This sample demonstrates strong security on an eb506 module.

Important: Please note this sample changes the passkey on the eb506
            to use a more secure passkey than the defualt (0000).  This
            sample uses a7blue*506 as a passkey, however, feel free to use
            whatever you want.  This passkey will be needed later to
            become trusted with the eb506.

Instructions
============
1. Verify that the eb506 is configured to use serial port F through
    jumpers J1 and J2.
2. Compile and run this program on the RabbitCore module using the
    "PROG" cable.
3. Disconnect the serial cable from the RabbitCore and recycle the power
    to the RCM prototyping board.
4. Toggle the security mode by pressing the switch, S2.
    - When visible is on and security is open, the LED will blink.
    - When visible is off and security is closed, the LED will be off.

    When the LED is blinking, another bluetooth device can become
    trusted with the eb506.  From this point on, these two devices
    will trust each other even when security is closed.  The idea is to
    open security, get trusted, then close security.

**************************************************************************/
#use a7eng_eb506.lib
#class auto

#define LED_DS1 6 // LED, port G bit 6
#define LED_DS2 7 // LED, port G bit 7
#define SWITCH_S2 1 // Switch, port G bit 1
#define SWITCH_S3 0 // Switch, port G bit 0
#define BAUD_RATE 9600L

int SetSecurityClosed()
{
  // Set visible off, security closed
  serFputs("set visible off\r");
  a7_serFwaitForString("ACK\r>", 0);
  serFputs("set security closed\r");
  a7_serFwaitForString("ACK\r>", 0);

  // Turn the LED off and return
  BitWrPortI(PGDR, &PGDRShadow, 1, LED_DS1);
  return 0;
}

int SetSecurityOpen()
{
  // Set visible on, security open
  serFputs("set visible on\r");
  a7_serFwaitForString("ACK\r>", 0);
  serFputs("set security open\r");
  a7_serFwaitForString("ACK\r>", 0);

  return 1;
}

void main()
{
  auto int nSwitchS2, nLEDDS1, nSecurity, nConnected, nBlinkRate;
  auto unsigned long lStartTick;

  nLEDDS1 = 1;       // LED is off
  nBlinkRate = 500;  // Half second
  nConnected = 0;    // 0 = NOT connected, 1 = connected
  nSecurity = 0;     // 0 = visible off, security closed
                     // 1 = visible on, security open

  // Open the serial port for communications and initialize the eb506
  serFopen(BAUD_RATE);
  a7_serFeb506Init();

  // Initialize the LEDs
  WrPortI(PGDDR, &PGDDRShadow, 0xC0 | PGDDRShadow);
  BitWrPortI(PGDR, &PGDRShadow, 1, LED_DS1);
  BitWrPortI(PGDR, &PGDRShadow, 1, LED_DS2);

  // Initialize the switch
  nSwitchS2 = BitRdPortI(PGDR, SWITCH_S2);

  // Use a secure passkey
  serFputs("set passkey a7blue*506\r");
  a7_serFwaitForString("ACK\r>", 0);

  // Use encryption
  serFputs("set encrypt on\r");
  a7_serFwaitForString("ACK\r>", 0);

  // Start with security closed
  nSecurity = SetSecurityClosed();

  while(1)
  {
    // Check for changes in the switch state
    if(BitRdPortI(PGDR, SWITCH_S2) ^ nSwitchS2)
    {
      nSwitchS2 = BitRdPortI(PGDR, SWITCH_S2);

      // Toggle the security mode when the switch is pressed
      if(nSwitchS2 && nConnected == 0)
      {
        if(nSecurity == 0)
        {
          nSecurity = SetSecurityOpen();

          // Setup for making the LED blink
          nLEDDS1 = 0;
          lStartTick = MS_TIMER;
        }
        else
        {
          nSecurity = SetSecurityClosed();
        }
      }
    }

    if(nSecurity == 1 && nConnected == 0)
    {
      // Make the LED blink to show we are have
      // visible on, security open
      if((MS_TIMER - lStartTick) < nBlinkRate)
      {
        if(nLEDDS1 == 0)
          nLEDDS1 = 1;
        else
          nLEDDS1 = 0;

        lStartTick += nBlinkRate;
      }

      BitWrPortI(PGDR, &PGDRShadow, nLEDDS1, LED_DS1);
    }

    // Ensure the prototyping board LED, DS1, mimics
    // the connection LED of the eb506
    if(BitRdPortI(PDDR, 4))
    {
      if(nConnected == 0)
      {
        // State change from disconnected to connected
        nConnected = 1;
        BitWrPortI(PGDR, &PGDRShadow, 0, LED_DS1);
      }
    }
    else
    {
      if(nConnected == 1)
      {
        // State change from connected to disconnected
        nConnected = 0;
        BitWrPortI(PGDR, &PGDRShadow, 1, LED_DS1);

        // Return to closed security after lost connection
        nSecurity = SetSecurityClosed();
      }
    }
  }

  serFclose();
}