//
//	Helper API Library for the eb501 on the PIC platform
//
//	File:			eb501PICLib.c
//	Date:			August 3, 2006
//	Hardware:		PICDEM 2 PLUS demo board, PIC18F452 controller, eb501 serial Bluetooth module
//	Description:	This library implements some utility and convinience functions for use by the
//					sample applications or others. Most of the functions are related to UART operation
//					and string handling. 
//	Notes:			Some features are left unimplemented.
//
//	Copyright 2005-2006 A7 Engineering Inc. 
//	Permission is granted to freely copy, modify, or redistribute this software.
//

// --- Includes ---

#include "eb501PICLib.h"

// --- Config ---

// Set configuration bits for use with ICD2 / PICDEM2 PLUS Demo Board:
//  - set HS oscillator
//  - disable watchdog timer
//  - disable low voltage programming
//  - enable background debugging
#pragma config OSC = HS
#pragma config WDT = OFF
#pragma config LVP = OFF
#pragma config DEBUG = ON

// --- Prototypes ---

extern void UARTOpenX(int config, int baud);

// --- Externals ---
 
extern union USART USART_Status;

// --- Globals ---


// --- Functions ---

void eb501Init(void)
{
  	// Perform any eb501 specific initialization	
	// Currently empty
}

void UARTOpen(int baud)
{
	int nBRG = 0;

	// To calculate the BRG value, use [ BRG = ((Fosc/baud)/64)-1 ]
	// hence with Nbaud = 9600bps, we get BRG = 25.042 ~= 25
	nBRG = ((FREQ_OSC/baud)/64)-1;

	// Open the USART configured as baud/8/N/1 in polled mode for transmit, and interrupt-based receive
  	UARTOpenX(USART_TX_INT_OFF&USART_RX_INT_ON&USART_ASYNCH_MODE&USART_EIGHT_BIT&USART_CONT_RX&USART_BRGH_HIGH, nBRG);
}

void UARTClose(void)
{
	// Disable the USART receiver
	RCSTA &= 0b01001111;

	// Disable the USART transmitter
	TXSTAbits.TXEN = 0;   

	// Disable both interrupts finally
	PIE1 &= 0b11001111;   // Disable both interrupts
}

void UARTOpenX(int config, int baud)
{
	// Reset the USART registers to POR state
	TXSTA = 0;           
	RCSTA = 0;

	// Do we want Sync or Async operation?
	if(config&0x01)      
	{
		TXSTAbits.SYNC = 1;
	}

	// Do we want 8 or 9 bit mode ?
	if(config&0x02)     
	{
		TXSTAbits.TX9 = 1;
		RCSTAbits.RX9 = 1;
	}
	
	// Do we want Master or Slave mode (sync only) ?
	if(config&0x04)
	{
		TXSTAbits.CSRC = 1;
	}

	// Do we want  Continuous or Single reception
	if(config&0x08)  
	{
		RCSTAbits.CREN = 1;
	}
	else
	{
		RCSTAbits.SREN = 1;
	}

	// Baud rate select (asychronous mode only) ?
	if(config&0x10)      
	{
		TXSTAbits.BRGH = 1;
	}
	else
	{
		TXSTAbits.BRGH = 0;
	}

	// Set the transmit interface bit
	PIR1bits.TXIF = 0;

	// Do we want an interrupt on receipt
	if(config&0x40)      
	{
		PIE1bits.RCIE = 1;
	}
	else
	{
		PIE1bits.RCIE = 0;
	}

	// Set the receive interface bit
	PIR1bits.RCIF = 0;

	// Do we want an interrupt on transmission
	if(config&0x80)  
	{
		PIE1bits.TXIE = 1;
	}
	else
	{
		PIE1bits.TXIE = 0;
	}

	// Write baudrate to SPBRG1
	SPBRG = baud;       

	// Enable the tranmsiiter
	TXSTAbits.TXEN = 1; 

	// Enable the receiver
	RCSTAbits.SPEN = 1;  
}

char UARTIsBusy(void)
{
	// If TRMT=1, the transmit shift register is empty
	if(TXSTAbits.TRMT)  
	{
		return 0;          
	}

	return 1;            
}

char UARTReadByte()
{
	char data;   

	// Clear previous status flags
	USART_Status.val &= 0xf2;          

	// Check for 9-bit mode
	if(RCSTAbits.RX9)                 
	{
		USART_Status.RX_NINE = 0;        
		if(RCSTAbits.RX9D)              
		{
			USART_Status.RX_NINE = 1;
		}
	}

	// Check for framing error 
	if(RCSTAbits.FERR)        
	{
		USART_Status.FRAME_ERROR = 1;    
	}

	// Check for overrun condition
	if(RCSTAbits.OERR)                 
	{
		USART_Status.OVERRUN_ERROR = 1;  
	}

	// Read the data
	data = RCREG;                      

	return data;                    
}

void UARTWriteByte(char c)
{
	putcUSART(c);
}

int UARTReadString(char* buf, int size, int timeout)
{
	int i = 0;
	unsigned char c = 0x0;

	if(!buf || !size || !timeout)
	{
		return 0;
	}

  	for(i=0; i<size; i++)  
  	{
		// Wait for data to be received
	    while(!DataRdyUSART());
	
	    // Get a character from the USART
		c = getcUSART();    
	    *buf = c;
	
		// Increment the string pointer
	    buf++;              
  	}

	return i;
}

void UARTWriteString(char* data)
{
	const rom far char* data2 = data;

	if(!data2)
	{
		return;
	}

	// Transmit the entire string, but not the NULL terminator
	while(*data2)
  	{
    	while(BusyUSART());
    	putcUSART(*data2);
		data2++;
  	} 
}

void UARTWriteBuffer(char* data, int nSize)
{
	if(!data || !nSize)
	{
		return;
	}

	// Transmit all the bytes in the buffer
  	while(nSize--)
  	{  
    	while(BusyUSART());
    	putcUSART(*data);
		data++;
  	} 	
}

int UARTWaitForString(char* msg, int timeout)
{
	char c = 0x0;
	int nSize = 0;
	int nCount = timeout*10;

	if(!msg || !timeout)
	{
		return 0;
	}

	while(msg[nSize] && nCount--)
	{
		// Wait for data to be received
	    while(!DataRdyUSART());
		
		// Read the char
		c = getcUSART();

#if !IGNORE_LF_NULL
		// Skip NULLs and LFs
		if(c == 0x0 || c == 0xA)
		{
			continue;
		}
#endif
		// Check for a match
		if(c == msg[nSize])
		{
			nSize++;
		}
		else
		{
			return 0;
		}
	}

	return nSize;
}

char PortBRead(void)
{
	return PORTB;
}

void PortBWrite(char value)
{
  	// Configure all PORTB pins for output
  	TRISB = 0;

	// Write the value
	PORTB = value & 0xFF;
}

void DelayMs(int pause)
{
	int nLoop = 0;
	
	// This function is not hyper-accurate, but fairly so
	// No ICD attached:
	// 4000 clock cycles is approximately 1ms
	// Calculate the number of 1K TCY to wait
	// and loop to get the total amount
	// With ICD attached:
	// The device is much slower, therefore
	// it works out to be about 1000 clock cycles 
	// of delay are required for a 1ms delay
#if USING_ICD_DEBUG
	nLoop = pause;
#else
	nLoop = 4*pause;
#endif

	for(nLoop; nLoop>0; nLoop--)
	{
		Delay1KTCYx(1);
	}
}
