//
//	Helper API Library for the eb501 on the PIC platform
//
//	File:			eb501PICLib.h
//	Date:			August 3, 2006
//	Hardware:		PICDEM 2 PLUS demo board, PIC18F452 controller, eb501 serial Bluetooth module
//	Description:	This library implements some utility and convinience functions for use by the
//					sample applications or others. Most of the functions are related to UART operation
//					and string handling. 
//	Notes:			Some features are left unimplemented. 
//
//	Copyright 2005-2006 A7 Engineering Inc. 
//	Permission is granted to freely copy, modify, or redistribute this software.
//

#ifndef __EB_501_PIC_LIB_H__
#define __EB_501_PIC_LIB_H__

// --- Includes ---

#include <p18f452.h>
#include <portb.h>
#include <usart.h>
#include <stdio.h>

// --- Config ---

// Set configuration bits for use with ICD2 / PICDEM2 PLUS Demo Board:
//  - set HS oscillator
//  - disable watchdog timer
//  - disable low voltage programming
//  - enable background debugging
//
//	This is done in eb501PICLib.c
// 

// --- Defines ---

// The frequency of the internal oscillator in Hz
#define FREQ_OSC			16000000

// Set to 1 if using the MPLAB ICD debugger
#define USING_ICD_DEBUG		1

// Set to 1 if you wish to ignore NULL and LF
// characters when waiting for strings
#define IGNORE_LF_NULL		1

// --- Public Interface Prototpyes

//
//	Function		
//		eb501Init
//
//	Paramaters		
//		None
//
//	Return
//		None
//
//	Description	
//		This function can be used to perform any eb501 specific initialization
//
void eb501Init(void);

//
//	Function		
//		UARTOpen
//
//	Paramaters		
//		baud	- the required baud rate
//
//	Return
//		None
//
//	Description	
//		This function will initialize the USART to operate in 8/N/1 mode at
//		the requested baud rate. Note that higher baud rates will result in
//		greater skews in timing due to BRG limitations
//
void UARTOpen(int baud);

//
//	Function		
//		UARTClose
//
//	Paramaters		
//		None
//
//	Return
//		None
//
//	Description	
//		This function will disable the USART and clear any data. It does
//		not flush the FIFOs however.
//
void UARTClose(void);

//
//	Function		
//		UARTReadByte
//
//	Paramaters		
//		None
//
//	Return
//		char	- the character read
//
//	Description	
//		This function will read 8 bits from the USART
//
char UARTReadByte(void);

//
//	Function		
//		UARTWriteByte
//
//	Paramaters		
//		cByte	- the byte to write 
//
//	Return
//		None
//
//	Description	
//		This function will write 8 bits to the USART
//
void UARTWriteByte(char cByte);

//
//	Function		
//		UARTReadString
//
//	Paramaters		
//		szBuffer	- a character buffer to store the read string
//		nSize		- the number of characters to read
//		nTimeout	- a timeout value in 'ticks'
//
//	Return
//		int		- the number of characters actually read 
//
//	Description	
//		This function will attempt to read the requested number of
//		bytes in the requested amount of time, and place the bytes 
//		in the output buffer.
//
int UARTReadString(char* szBuffer, int nSize, int nTimeout);

//
//	Function		
//		UARTWriteString
//
//	Paramaters		
//		szData		- a comma-delimited string (ROM memory)
//
//	Return
//		None
//
//	Description	
//		This function will write to the USART a comma-delimited
//		ASCII string which is stored in ROM address space. It does
//		not transmit the trailing NULL. This function should be
//		used to transmit strings, and not byte buffers (use the 
//		function 'UARTWriteBuffer' for that).
//
void UARTWriteString(char* szData);

//
//	Function		
//		UARTWriteBuffer
//
//	Paramaters		
//		pbData	- a byte buffer (program memory)
//		nSize	- the number of bytes to write
//
//	Return
//		None
//
//	Description	
//		This function will write to the USART the requested
//		number of bytes from the provided byte buffer.
//
void UARTWriteBuffer(char* pbData, int nSize);

//
//	Function		
//		UARTWaitForString
//
//	Paramaters		
//		szData		- a character buffer containing the string to wait for (program memory) 
//		nTimeout	- a timeout value in 'ticks' 
//
//	Return
//		int 		- a value of 0 indicates failure, otherwise the number of bytes read
//
//	Description	
//		This function will wait for the specified string to be received 
//		from the USART. If the data received does not match, a failure
//		is returned. Note that it does not check bytes received after
//		a match is found. Also, it can be set to ignore NULL and LF 
//		characters. Currently, the timeout feature is unimplemented.
//
int UARTWaitForString(char* szData, int nTimeout);

//
//	Function		
//		PortBRead
//
//	Paramaters		
//		None
//
//	Return
//		char	- the 8-bit value of Port B
//
//	Description	
//		This function reads PORTB.
//
char PortBRead(void);

//
//	Function		
//		PortBWrite
//
//	Paramaters		
//		cValue	- the 8-bit value to write to Port B
//
//	Return
//		None
//
//	Description	
//		This function writes PORTB.
//
void PortBWrite(char cValue);

//
//	Function		
//		DelayMs
//
//	Paramaters		
//		nDelay	- the number of milliseconds to delay
//
//	Return
//		None
//
//	Description	
//		This function delays the currently executing thread's
//		mainline by the specified number of milliseconds.
//
void DelayMs(int nDelay);

#endif //__EB_501_PIC_LIB_H__

